
# q-expansion of newform 1200.3.c.c, downloaded from the LMFDB on 06 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1200
weight = 3
poly_data = [1, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[751, [1, 0]], [901, [1, 0]], [401, [-1, 0]], [577, [-1, 0]]]
aps_data = [[0, 0], [0, 3], [0, 0], [0, 2], [0, 0], [0, -22], [0, 0], [26, 0], [0, 0], [0, 0], [46, 0], [0, -26], [0, 0], [0, 22], [0, 0], [0, 0], [0, 0], [74, 0], [0, 122], [0, 0], [0, -46], [-142, 0], [0, 0], [0, 0], [0, -2], [0, 0], [0, -194], [0, 0], [214, 0], [0, 0], [0, 146], [0, 0], [0, 0], [-22, 0], [0, 0], [286, 0], [0, 118], [0, 262], [0, 0], [0, 0], [0, 0], [314, 0], [0, 0], [0, -382], [0, 0], [386, 0], [166, 0], [0, -338], [0, 0], [-26, 0], [0, 0], [0, 0], [-286, 0], [0, 0], [0, 0], [0, 0], [0, 0], [-242, 0], [0, -122], [0, 0], [0, -458], [0, 0], [0, -358], [0, 0], [0, -142], [0, 0], [-362, 0], [0, -482], [0, 0], [502, 0], [0, 0], [0, 0], [0, -718], [0, 698], [-694, 0], [0, 0], [0, 0], [0, -362], [0, 0], [-626, 0], [0, 0], [-358, 0], [0, 0], [0, -862], [-94, 0], [0, 0], [0, 0], [0, 814], [0, 0], [0, 526], [0, 0], [0, 0], [0, 962], [0, 0], [26, 0], [0, 0], [0, 0], [0, 0], [0, 982], [1034, 0], [0, 506], [0, 0], [0, 0], [0, 0], [886, 0], [0, -962], [0, 0], [0, 0], [0, 0], [-526, 0], [0, -814], [0, -1126], [0, 0], [-214, 0], [-674, 0], [0, 0], [0, -314], [0, 0], [0, 0], [0, 0], [122, 0], [0, 1154], [0, 0], [0, 0], [1318, 0], [0, 0], [934, 0], [0, 0], [0, 482], [0, 1034], [-1222, 0], [0, 0], [-1202, 0], [0, 838], [0, 0], [1534, 0], [0, 0], [0, 1562], [0, 0], [0, 0], [-1514, 0], [0, 0], [0, -1058], [0, 0], [-458, 0], [0, 0], [0, 1658], [0, 0], [1418, 0], [0, 0], [0, 598], [0, 0], [0, 1702], [0, 0], [0, -214], [0, 0], [866, 0], [0, 0], [0, 1198], [0, 0], [0, 0], [0, 0], [0, -1534], [0, 0], [0, 0], [0, 0], [46, 0], [0, 1894], [-1826, 0], [0, 0], [0, 0], [-1846, 0], [0, 0], [0, 1298], [626, 0], [0, 0], [-1994, 0], [0, 0], [0, 1342], [-1706, 0], [0, -2158], [0, 0], [0, -1702], [0, 0], [0, 0], [0, 0], [0, 118], [0, 1222], [814, 0], [0, 0], [0, 1538], [0, 0], [-1754, 0], [0, 0], [0, 0], [0, 0], [-2398, 0], [0, 74], [0, 0], [0, 0], [0, 0], [-2162, 0], [0, -2426], [2302, 0], [0, 0], [0, 0], [-1774, 0], [0, 0], [0, 0], [-554, 0], [0, 478], [0, 0], [0, -2018], [0, 0], [0, 0], [-2158, 0], [0, -2638], [0, 0], [0, 0], [0, 0], [2714, 0], [1826, 0], [0, 0], [0, 2446], [0, 0], [-506, 0], [0, 0], [0, 0], [0, 2882], [0, 0], [0, -2902], [218, 0], [-2834, 0], [0, 0], [0, 1366], [0, 0], [1822, 0], [0, 0], [0, 0], [0, 0], [0, 0], [-1034, 0], [0, 382], [-746, 0], [0, 0], [0, 0], [0, -1198], [0, 0], [-2134, 0], [0, 0], [0, 694], [0, 0], [0, 0], [-146, 0], [0, 0], [0, 0], [-2566, 0], [0, 3146], [0, 0], [0, -1586], [0, -1298], [0, 0], [-2138, 0], [0, 3338], [0, 0], [698, 0], [0, 0], [0, 0], [0, 1846], [0, 0], [-2326, 0], [0, 2906], [0, -3406], [-814, 0], [0, 3358], [0, 2782], [0, 0], [-3146, 0], [1874, 0], [0, 0], [0, 0], [-962, 0], [0, 0], [3674, 0], [0, -598], [0, 0], [0, 2978], [0, 0], [-3742, 0], [0, 0], [0, 0], [0, 0], [0, 0], [0, 0], [0, -22], [0, 0], [-1874, 0], [0, 0], [0, 0], [0, -2374], [0, 914], [0, 0], [-1294, 0]]
